"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Selection = void 0;
var reactive_1 = require("@formily/reactive");
var events_1 = require("../events");
var shared_1 = require("@designable/shared");
var Selection = /** @class */ (function () {
    function Selection(props) {
        this.selected = [];
        if (props.selected) {
            this.selected = props.selected;
        }
        if (props.operation) {
            this.operation = props.operation;
        }
        this.makeObservable();
    }
    Selection.prototype.makeObservable = function () {
        reactive_1.define(this, {
            selected: reactive_1.observable,
            select: reactive_1.action,
            batchSelect: reactive_1.action,
            add: reactive_1.action,
            remove: reactive_1.action,
            clear: reactive_1.action,
            crossAddTo: reactive_1.action,
        });
    };
    Selection.prototype.trigger = function (type, fromUser) {
        if (type === void 0) { type = events_1.SelectNodeEvent; }
        if (fromUser === void 0) { fromUser = false; }
        return this.operation.dispatch(new type({
            target: this.operation.tree,
            source: this.operation.getSelectedNodes(),
            extra: {
                fromUser: fromUser,
            },
        }));
    };
    Selection.prototype.select = function (id, fromUser) {
        if (shared_1.isStr(id)) {
            if (this.selected.length === 1 && this.selected.includes(id))
                return;
            this.selected = [id];
            this.trigger(events_1.SelectNodeEvent, fromUser);
        }
        else {
            this.select(id === null || id === void 0 ? void 0 : id.id, fromUser);
        }
    };
    Selection.prototype.safeSelect = function (id, fromUser) {
        if (!id)
            return;
        this.select(id, fromUser);
    };
    Selection.prototype.mapIds = function (ids) {
        return shared_1.isArr(ids)
            ? ids.map(function (node) { return (shared_1.isStr(node) ? node : node === null || node === void 0 ? void 0 : node.id); })
            : [];
    };
    Selection.prototype.batchSelect = function (ids, fromUser) {
        this.selected = this.mapIds(ids);
        this.trigger(events_1.SelectNodeEvent, fromUser);
    };
    Selection.prototype.batchSafeSelect = function (ids, fromUser) {
        if (!(ids === null || ids === void 0 ? void 0 : ids.length))
            return;
        this.batchSelect(ids, fromUser);
    };
    Object.defineProperty(Selection.prototype, "first", {
        get: function () {
            if (this.selected && this.selected.length)
                return this.selected[0];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Selection.prototype, "last", {
        get: function () {
            if (this.selected && this.selected.length)
                return this.selected[this.selected.length - 1];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Selection.prototype, "length", {
        get: function () {
            return this.selected.length;
        },
        enumerable: false,
        configurable: true
    });
    Selection.prototype.add = function () {
        var _this = this;
        var ids = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            ids[_i] = arguments[_i];
        }
        this.mapIds(ids).forEach(function (id) {
            if (shared_1.isStr(id)) {
                if (!_this.selected.includes(id)) {
                    _this.selected.push(id);
                }
            }
            else {
                _this.add(id === null || id === void 0 ? void 0 : id.id);
            }
        });
        this.trigger();
    };
    Selection.prototype.crossAddTo = function (node) {
        var _this = this;
        if (node.parent) {
            var selectedNodes = this.operation.getSelectedNodes();
            if (this.has(node)) {
                this.remove(node);
            }
            else {
                var minDistanceNode = selectedNodes.reduce(function (minDistanceNode, item) {
                    return item.distanceTo(node) < minDistanceNode.distanceTo(node)
                        ? item
                        : minDistanceNode;
                }, selectedNodes[0]);
                if (minDistanceNode) {
                    var crossNodes = node.crossSiblings(minDistanceNode);
                    crossNodes.forEach(function (node) {
                        if (!_this.selected.includes(node.id)) {
                            _this.selected.push(node.id);
                        }
                    });
                }
                if (!this.selected.includes(node.id)) {
                    this.selected.push(node.id);
                }
            }
        }
    };
    Selection.prototype.remove = function () {
        var _this = this;
        var ids = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            ids[_i] = arguments[_i];
        }
        this.mapIds(ids).forEach(function (id) {
            if (shared_1.isStr(id)) {
                _this.selected = _this.selected.filter(function (item) { return item !== id; });
            }
            else {
                _this.remove(id === null || id === void 0 ? void 0 : id.id);
            }
        });
        this.trigger(events_1.UnSelectNodeEvent);
    };
    Selection.prototype.has = function () {
        var _this = this;
        var ids = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            ids[_i] = arguments[_i];
        }
        return this.mapIds(ids).some(function (id) {
            if (shared_1.isStr(id)) {
                return _this.selected.includes(id);
            }
            else {
                if (!(id === null || id === void 0 ? void 0 : id.id))
                    return false;
                return _this.has(id === null || id === void 0 ? void 0 : id.id);
            }
        });
    };
    Selection.prototype.clear = function () {
        this.selected = [];
        this.trigger(events_1.UnSelectNodeEvent);
    };
    return Selection;
}());
exports.Selection = Selection;
